#/*
# * ATC - Air Traffic Controller simulation game
# *
# * Copyright (C) 2004 Josef "Jeff" Sipek <jeffpc@optonline.net>
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# *
# * @(#) atc_plane.py 1.3 04/02/20 13:03:10
# */

import os
import sys
import pygame
from math import *

import atc_colors
import atc_utils

mperpix		= 750.0

def m2pix(m):
	""" Meters to pixels conversion fn """
	return m/mperpix

def pix2m(pix):
	""" Pixels to meters conversion fn """
	return pix*mperpix

plane_OK	= 0
plane_RANGE	= 1
plane_CRASHED	= 2
plane_STALL	= 3
plane_DEAD	= 4
plane_PROXY	= 5

plane_PROP	= 16.667 # meters/sec
plane_JET	= 62.572


class Plane(pygame.sprite.Sprite):
	""" Class to manage one plane's motion """
	def __init__(self,callsign,pos=(0.0, 0.0, 0.0),vel=(0.0, 0.0, 0.0)):
		""" Set up everything """
		pygame.sprite.Sprite.__init__(self)
		
		# screen
		screen = pygame.display.get_surface()
		self.area = screen.get_rect()
		
		# plane image
		(self.image, self.rect) = atc_utils.load_png('data/image/plane.png')
		
		# call sign
		self.callsign	= callsign
		
		# position
		self.X		= pos[0]
		self.Y		= pos[1]
		self.Z		= pos[2]
		
		# velocity
		self.i		= vel[0]
		self.j		= vel[1]
		self.k		= vel[2]
		
		# plane specs
		self.stallspeed	= m2pix(plane_PROP)
		
		# status flag
		self.status	= plane_OK
	
	def update(self):
		""" Move the plane, and check for collisions """
		if (self.status != plane_OK):
			return
		
		self.X += self.i
		self.Y += self.j
		self.Z += self.k
		
		if (self.X <= 0) or (self.Y <= 0): # FIXME: max values
			self.status = plane_RANGE
		
		if (self.Z <= 0):
			self.i = 0
			self.j = 0
			self.k = 0
			self.status = plane_CRASHED
		
		if (self.calc_vel()<self.stallspeed) and (not self.status==plane_CRASHED):
			self.status = plane_STALL
		
		self.rect.move(self.i,self.j)
	
	def calc_vel(self):
		""" Calculate magnitude of the velocity """
		return sqrt(self.i**2 + self.j**2 + self.k**2)
	
	def calc_head(self):
		""" Calculate heading of the airplane """
		if (self.i == self.j == 0): # no movement
			return 0
		
		try:
			head = fabs(atan(self.i/-self.j))
		except ZeroDivisionError:
			if (self.i>0):
				return pi/2	# east
			return 3*pi/2		# west
		
		if (self.i>0) and (-self.j>0):	# east and north
			return pi/2 - head
		if (self.i<0) and (-self.j>0):	# west and north
			return head + 3*pi/2
		if (self.i>0) and (-self.j<0):	# east and south
			return head + pi/2
		if (self.i==0) and (-self.j>0):	# north
			return head
		return head + pi		# west and south / south
	
	def calc_rc(self):
		""" Calculate rate of climb """
		return atan(self.k/self.i)	# REDO!!
	
	def recalc_vel(self,vel,heading,rc):
		""" Set velocity to new value """
		self.i = vel*cos(rc)
		self.j = vel*cos(heading)
		self.k = vel*sin(rc)

	def display(self,screen):
		""" Put everything onto the screen """
		screen.blit(self.image, (int(self.X), int(self.Y)))
		
		# Plane info
		font = pygame.font.Font(None, 16)
		
		color = atc_colors.planeinfo_ok
		if (self.status == plane_CRASHED) or (self.status == plane_DEAD):
			color = atc_colors.planeinfo_crashed
		
		x = int(self.X) + 10
		y = int(self.Y) - 5
		
		strings = (	self.callsign + str(self.status),
				"Alt:  " + str(int(self.Z)),
				"Lat:  " + str(int(self.X)),
				"Long: " + str(int(self.Y)),
				"Head: " + str(int(atc_utils.todeg(self.calc_head()))))
		
		for stri in strings:
			alt = font.render(stri, 1, color)
			altpos = alt.get_rect()
			altpos.topleft = (x,y)
			screen.blit(alt, altpos)
			(x,y) = (x,y+10)
